@@ -241,6 +241,27 @@ class Bucket(AbstractBucket):
 
         return extractmeta(resp)
     
+    @retry
+    def get_size(self, key):
+        '''Return size of object stored under *key*'''
+        
+        log.debug('get_size(%s)', key)
+        
+        try:
+            resp = self._do_request('HEAD', '/%s%s' % (self.prefix, key))
+            assert resp.length == 0
+        except HTTPError as exc:
+            if exc.status == 404:
+                raise NoSuchObject(key)
+            else:
+                raise
+
+        for (name, val) in resp.getheaders():
+            if name.lower() == 'content-length':
+                return int(val)
+        raise RuntimeError('HEAD request did not return Content-Length')
+        
+            
     @retry
     def open_read(self, key):
         ''''Open object for reading
@@ -403,7 +424,9 @@ class Bucket(AbstractBucket):
         False.
         """
 
-        for (no, s3key) in enumerate(self):
+        # We have to cache keys, because otherwise we can't use the
+        # http connection to delete keys.
+        for (no, s3key) in enumerate(list(self)):
             if no != 0 and no % 1000 == 0:
                 log.info('clear(): deleted %d objects so far..', no)
 
@@ -521,6 +544,7 @@ class ObjectR(object):
             buf = self.read(BUFSIZE)
             if buf == '':
                 break
+            self.md5.update(buf)
 
         etag = self.resp.getheader('ETag').strip('"')
         log.debug('ObjectR(%s).close(): md5sum/etag: %s', self.key, etag)        
