@@ -7,22 +7,24 @@ This program can be distributed under the terms of the GNU GPLv3.
 '''
 
 from __future__ import division, print_function, absolute_import
+from cgitb import scanvars, __UNDEF__
 from llfuse import ROOT_INODE
-import cPickle as pickle
+import bz2
 import hashlib
+import inspect
+import linecache
 import logging
 import os
+import pydoc
 import stat
 import sys
-import tempfile
-import time
-import lzma
+import types
 
 # Buffer size when writing objects
 BUFSIZE = 256 * 1024
 
 log = logging.getLogger('common')
-        
+            
 def setup_logging(options):        
     root_logger = logging.getLogger()
     if root_logger.handlers:
@@ -124,64 +126,12 @@ def get_seq_no(bucket):
         
     return seq_no   
 
-def cycle_metadata(bucket):
-    from .backends.common import NoSuchObject
+def stream_write_bz2(ifh, ofh):
+    '''Compress *ifh* into *ofh* using bz2 compression'''
     
-    log.info('Backing up old metadata...')
-    for i in reversed(range(10)):
-        try:
-            bucket.copy("s3ql_metadata_bak_%d" % i, "s3ql_metadata_bak_%d" % (i + 1))
-        except NoSuchObject:
-            pass
-                
-    bucket.copy("s3ql_metadata", "s3ql_metadata_bak_0")             
-
-def dump_metadata(ofh, conn):
-    
-    log.info('Dumping metadata...')
-    # First write everything into temporary file
-    tmp = tempfile.TemporaryFile()
-    pickler = pickle.Pickler(tmp, 2)
-    bufsize = 256
-    buf = range(bufsize)
-    tables_to_dump = [('objects', 'id'), ('blocks', 'id'),
-                      ('inode_blocks', 'inode, blockno'),
-                      ('inodes', 'id'), ('symlink_targets', 'inode'),
-                      ('names', 'id'), ('contents', 'parent_inode, name_id'),
-                      ('ext_attributes', 'inode, name')]
-
-    columns = dict()
-    for (table, _) in tables_to_dump:
-        columns[table] = list()
-        for row in conn.query('PRAGMA table_info(%s)' % table):
-            columns[table].append(row[1])
-
-    pickler.dump((tables_to_dump, columns))
-    
-    for (table, order) in tables_to_dump:
-        log.info('..%s..' % table)
-        pickler.clear_memo()
-        i = 0
-        for row in conn.query('SELECT %s FROM %s ORDER BY %s' 
-                              % (','.join(columns[table]), table, order)):
-            buf[i] = row
-            i += 1
-            if i == bufsize:
-                pickler.dump(buf)
-                pickler.clear_memo()
-                i = 0
-
-        if i != 0:
-            pickler.dump(buf[:i])
-        
-        pickler.dump(None)
-
-    # Then compress and send
-    log.info("Compressing and uploading metadata...")
-    compr = lzma.LZMACompressor(options={ 'level': 7 })
-    tmp.seek(0)
+    compr = bz2.BZ2Compressor(9)
     while True:
-        buf = tmp.read(BUFSIZE)
+        buf = ifh.read(BUFSIZE)
         if not buf:
             break
         buf = compr.compress(buf)
@@ -190,45 +140,19 @@ def dump_metadata(ofh, conn):
     buf = compr.flush()
     if buf:
         ofh.write(buf)
-    del compr # Free memory ASAP, LZMA level 7 needs 186 MB
-    tmp.close()
 
-def restore_metadata(ifh, conn):
-
-    # Note: unpickling is terribly slow if fh is not a real file object, so
-    # uncompressing to a temporary file also gives a performance boost
-    log.info('Downloading and decompressing metadata...')
-    tmp = tempfile.TemporaryFile()
-    decompressor = lzma.LZMADecompressor()
+def stream_read_bz2(ifh, ofh):
+    '''Uncompress bz2 compressed *ifh* into *ofh*'''
+    
+    decompressor = bz2.BZ2Decompressor()
     while True:
         buf = ifh.read(BUFSIZE)
         if not buf:
             break
         buf = decompressor.decompress(buf)
         if buf:
-            tmp.write(buf)
-    del decompressor
-    tmp.seek(0) 
-
-    log.info("Reading metadata...")
-    unpickler = pickle.Unpickler(tmp)
-    (to_dump, columns) = unpickler.load()
-    create_tables(conn)
-    for (table, _) in to_dump:
-        log.info('..%s..', table)
-        col_str = ', '.join(columns[table])
-        val_str = ', '.join('?' for _ in columns[table])
-        sql_str = 'INSERT INTO %s (%s) VALUES(%s)' % (table, col_str, val_str)
-        while True:
-            buf = unpickler.load()
-            if not buf:
-                break
-            for row in buf:
-                conn.execute(sql_str, row)
-
-    tmp.close()
-    conn.execute('ANALYZE')
-    
+            ofh.write(buf)       
+                  
 class QuietError(Exception):
     '''
     QuietError is the base class for exceptions that should not result
@@ -246,6 +170,72 @@ class QuietError(Exception):
     def __str__(self):
         return self.msg
 
+# Adapted from cgitb.text, but less verbose
+def format_tb(einfo):
+    """Return a plain text document describing a given traceback."""
+    
+    etype, evalue, etb = einfo
+    if type(etype) is types.ClassType:
+        etype = etype.__name__
+
+    frames = [ 'Traceback (most recent call last):' ]
+    records = inspect.getinnerframes(etb, context=7)
+    for (frame, file_, lnum, func, lines, index) in records:
+        (args, varargs, varkw, locals_) = inspect.getargvalues(frame)
+        sig = inspect.formatargvalues(args, varargs, varkw, locals_,
+                                      formatvalue=lambda value: '=' + pydoc.text.repr(value))
+
+        rows = ['  File %r, line %d, in %s%s' % (file_, lnum, func, sig) ]
+        
+        # To print just current line
+        if index is not None:
+            rows.append('    %s' % lines[index].strip())
+            
+#        # To print with context:
+#        if index is not None:
+#            i = lnum - index
+#            for line in lines:
+#                num = '%5d ' % i
+#                rows.append(num+line.rstrip())
+#                i += 1
+                            
+        def reader(lnum=[lnum]): #pylint: disable=W0102 
+            try: 
+                return linecache.getline(file_, lnum[0])
+            finally:
+                lnum[0] += 1
+
+        printed = set()
+        rows.append('  Current bindings:')
+        for (name, where, value) in scanvars(reader, frame, locals_):
+            if name in printed: 
+                continue
+            printed.add(name)
+            if value is not __UNDEF__:
+                if where == 'global': 
+                    where = '(global)'
+                elif where != 'local': 
+                    name = where + name.split('.')[-1]
+                    where = '(local)'
+                else: 
+                    where = ''
+                rows.append('    %s = %s %s' % (name, pydoc.text.repr(value), where))
+            else:
+                rows.append(name + ' undefined')
+
+        rows.append('')
+        frames.extend(rows)
+
+    exception = ['Exception: %s: %s' % (etype.__name__, evalue)]
+    if isinstance(evalue, BaseException):
+        for name in dir(evalue):
+            if name.startswith('__'):
+                continue
+            value = pydoc.text.repr(getattr(evalue, name))
+            exception.append('  %s = %s' % (name, value))
+
+    return '%s\n%s' % ('\n'.join(frames), '\n'.join(exception))
+
 def setup_excepthook():
     '''Modify sys.excepthook to log exceptions
     
@@ -258,9 +248,14 @@ def setup_excepthook():
         if isinstance(val, QuietError):
             root_logger.error(val.msg)
         else:
-            root_logger.error('Uncaught top-level exception', 
-                              exc_info=(type_, val, tb))
-            
+            try:
+                msg = format_tb((type_, val, tb))
+            except:
+                root_logger.error('Uncaught top-level exception -- and tb handler failed!',
+                                  exc_info=(type_, val, tb))
+            else:        
+                root_logger.error('Uncaught top-level exception. %s', msg)
+                
     sys.excepthook = excepthook 
     
 def inode_for_path(path, conn):
@@ -352,123 +347,4 @@ def sha256_fh(fh):
 
     return sha.digest()
 
-def init_tables(conn):
-    # Insert root directory
-    timestamp = time.time() - time.timezone
-    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime,atime,ctime,refcount) "
-                 "VALUES (?,?,?,?,?,?,?,?)",
-                   (ROOT_INODE, stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-                   | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH,
-                    os.getuid(), os.getgid(), timestamp, timestamp, timestamp, 1))
-
-    # Insert control inode, the actual values don't matter that much 
-    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime,atime,ctime,refcount) "
-                 "VALUES (?,?,?,?,?,?,?,?)",
-                 (CTRL_INODE, stat.S_IFIFO | stat.S_IRUSR | stat.S_IWUSR,
-                  0, 0, timestamp, timestamp, timestamp, 42))
-
-    # Insert lost+found directory
-    inode = conn.rowid("INSERT INTO inodes (mode,uid,gid,mtime,atime,ctime,refcount) "
-                       "VALUES (?,?,?,?,?,?,?)",
-                       (stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR,
-                        os.getuid(), os.getgid(), timestamp, timestamp, timestamp, 1))
-    name_id = conn.rowid('INSERT INTO names (name, refcount) VALUES(?,?)',
-                         (b'lost+found', 1))
-    conn.execute("INSERT INTO contents (name_id, inode, parent_inode) VALUES(?,?,?)",
-                 (name_id, inode, ROOT_INODE))
-
-def create_tables(conn): 
-    # Table of storage objects
-    # Refcount is included for performance reasons
-    conn.execute("""
-    CREATE TABLE objects (
-        id        INTEGER PRIMARY KEY AUTOINCREMENT,
-        refcount  INT NOT NULL, 
-        size      INT  
-    )""")
-
-    # Table of known data blocks
-    # Refcount is included for performance reasons
-    conn.execute("""
-    CREATE TABLE blocks (
-        id        INTEGER PRIMARY KEY,
-        hash      BLOB(16) UNIQUE,
-        refcount  INT NOT NULL,
-        size      INT NOT NULL,    
-        obj_id    INTEGER NOT NULL REFERENCES objects(id)
-    )""")
-                
-    # Table with filesystem metadata
-    # The number of links `refcount` to an inode can in theory
-    # be determined from the `contents` table. However, managing
-    # this separately should be significantly faster (the information
-    # is required for every getattr!)
-    conn.execute("""
-    CREATE TABLE inodes (
-        -- id has to specified *exactly* as follows to become
-        -- an alias for the rowid.
-        id        INTEGER PRIMARY KEY,
-        uid       INT NOT NULL,
-        gid       INT NOT NULL,
-        mode      INT NOT NULL,
-        mtime     REAL NOT NULL,
-        atime     REAL NOT NULL,
-        ctime     REAL NOT NULL,
-        refcount  INT NOT NULL,
-        size      INT NOT NULL DEFAULT 0,
-        rdev      INT NOT NULL DEFAULT 0,
-        locked    BOOLEAN NOT NULL DEFAULT 0
-    )""")
-
-    # Further Blocks used by inode (blockno >= 1)
-    conn.execute("""
-    CREATE TABLE inode_blocks (
-        inode     INTEGER NOT NULL REFERENCES inodes(id),
-        blockno   INT NOT NULL,
-        block_id    INTEGER NOT NULL REFERENCES blocks(id),
-        PRIMARY KEY (inode, blockno)
-    )""")
-    
-    # Symlinks
-    conn.execute("""
-    CREATE TABLE symlink_targets (
-        inode     INTEGER PRIMARY KEY REFERENCES inodes(id),
-        target    BLOB NOT NULL
-    )""")
-    
-    # Names of file system objects
-    conn.execute("""
-    CREATE TABLE names (
-        id     INTEGER PRIMARY KEY,
-        name   BLOB NOT NULL,
-        refcount  INT NOT NULL,
-        UNIQUE (name)
-    )""")
-
-    # Table of filesystem objects
-    # rowid is used by readdir() to restart at the correct position
-    conn.execute("""
-    CREATE TABLE contents (
-        rowid     INTEGER PRIMARY KEY AUTOINCREMENT,
-        name_id   INT NOT NULL REFERENCES names(id),
-        inode     INT NOT NULL REFERENCES inodes(id),
-        parent_inode INT NOT NULL REFERENCES inodes(id),
-        
-        UNIQUE (parent_inode, name_id)
-    )""")
-
-    # Extended attributes
-    conn.execute("""
-    CREATE TABLE ext_attributes (
-        inode     INTEGER NOT NULL REFERENCES inodes(id),
-        name      BLOB NOT NULL,
-        value     BLOB NOT NULL,
- 
-        PRIMARY KEY (inode, name)               
-    )""")
-
-    # Shortcurts
-    conn.execute("""
-    CREATE VIEW contents_v AS
-    SELECT * FROM contents JOIN names ON names.id = name_id       
-    """)    
\ No newline at end of file
+    
\ No newline at end of file
