@@ -10,7 +10,8 @@ from __future__ import division, print_function, absolute_import
 from .backends.common import NoSuchObject, ChecksumError
 from .common import (get_path, CTRL_NAME, CTRL_INODE, LoggerFilter)
 from .database import NoSuchRowError
-from .inode_cache import InodeCache, OutOfInodesError
+from .inode_cache import OutOfInodesError
+from . import deltadump
 from cStringIO import StringIO
 from llfuse import FUSEError
 import cPickle as pickle
@@ -68,10 +69,11 @@ class Operations(llfuse.Operations):
     explicitly checks the st_mode attribute.
     """
         
-    def __init__(self, block_cache, db, blocksize, upload_event=None):
+    def __init__(self, block_cache, db, blocksize, inode_cache,
+                 upload_event=None):
         super(Operations, self).__init__()
 
-        self.inodes = InodeCache(db)
+        self.inodes = inode_cache
         self.db = db
         self.upload_event = upload_event
         self.open_inodes = collections.defaultdict(lambda: 0)
@@ -172,7 +174,7 @@ class Operations(llfuse.Operations):
 
         else:
             try:
-                value = self.db.get_val('SELECT value FROM ext_attributes WHERE inode=? AND name=?',
+                value = self.db.get_val('SELECT value FROM ext_attributes_v WHERE inode=? AND name=?',
                                           (id_, name))
             except NoSuchRowError:
                 raise llfuse.FUSEError(llfuse.ENOATTR)
@@ -180,7 +182,7 @@ class Operations(llfuse.Operations):
 
     def listxattr(self, id_):
         names = list()
-        for (name,) in self.db.query('SELECT name FROM ext_attributes WHERE inode=?', (id_,)):
+        for (name,) in self.db.query('SELECT name FROM ext_attributes_v WHERE inode=?', (id_,)):
             names.append(name)
         return names
 
@@ -211,8 +213,11 @@ class Operations(llfuse.Operations):
             if self.inodes[id_].locked:
                 raise FUSEError(errno.EPERM)
                     
-            self.db.execute('INSERT OR REPLACE INTO ext_attributes (inode, name, value) '
-                            'VALUES(?, ?, ?)', (id_, name, value))
+            if len(value) > deltadump.MAX_BLOB_SIZE:
+                raise FUSEError(errno.EINVAL)
+            
+            self.db.execute('INSERT OR REPLACE INTO ext_attributes (inode, name_id, value) '
+                            'VALUES(?, ?, ?)', (id_, self._add_name(name), value))
             self.inodes[id_].ctime = time.time()
 
     def removexattr(self, id_, name):
@@ -220,10 +225,16 @@ class Operations(llfuse.Operations):
         if self.inodes[id_].locked:
             raise FUSEError(errno.EPERM)
             
-        changes = self.db.execute('DELETE FROM ext_attributes WHERE inode=? AND name=?',
-                                  (id_, name))
+        try:
+            name_id =  self._del_name(name)
+        except NoSuchRowError:
+            raise llfuse.FUSEError(llfuse.ENOATTR)
+        
+        changes = self.db.execute('DELETE FROM ext_attributes WHERE inode=? AND name_id=?',
+                                  (id_, name_id))
         if changes == 0:
             raise llfuse.FUSEError(llfuse.ENOATTR)
+        
         self.inodes[id_].ctime = time.time()
 
     def lock_tree(self, id0):
@@ -371,14 +382,21 @@ class Operations(llfuse.Operations):
                     db.execute('INSERT INTO symlink_targets (inode, target) '
                                'SELECT ?, target FROM symlink_targets WHERE inode=?',
                                (id_new, id_))
-                        
+
+                    db.execute('INSERT INTO ext_attributes (inode, name_id, value) '
+                               'SELECT ?, name_id, value FROM ext_attributes WHERE inode=?',
+                               (id_new, id_))
+                    db.execute('UPDATE names SET refcount = refcount + 1 WHERE '
+                               'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)',
+                               (id_,))
+                                            
                     processed += db.execute('INSERT INTO inode_blocks (inode, blockno, block_id) '
                                             'SELECT ?, blockno, block_id FROM inode_blocks '
                                             'WHERE inode=?', (id_new, id_))
                     db.execute('REPLACE INTO blocks (id, hash, refcount, size, obj_id) '
-                               'SELECT id, hash, refcount+1, size, obj_id '
+                               'SELECT id, hash, refcount+COUNT(id), size, obj_id '
                                'FROM inode_blocks JOIN blocks ON block_id = id '
-                               'WHERE inode = ?', (id_new,))
+                               'WHERE inode = ? GROUP BY id', (id_new,))
                     
                     if db.has_val('SELECT 1 FROM contents WHERE parent_inode=?', (id_,)):
                         queue.append((id_, id_new, 0))
@@ -474,6 +492,10 @@ class Operations(llfuse.Operations):
             self.cache.remove(id_, 0, int(math.ceil(inode.size / self.blocksize)))
             # Since the inode is not open, it's not possible that new blocks
             # get created at this point and we can safely delete the inode
+            self.db.execute('UPDATE names SET refcount = refcount - 1 WHERE '
+                            'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
+                            (id_,))
+            self.db.execute('DELETE FROM names WHERE refcount=0')                 
             self.db.execute('DELETE FROM ext_attributes WHERE inode=?', (id_,))
             self.db.execute('DELETE FROM symlink_targets WHERE inode=?', (id_,))
             del self.inodes[id_]
@@ -613,6 +635,10 @@ class Operations(llfuse.Operations):
                               int(math.ceil(inode_new.size / self.blocksize)))
             # Since the inode is not open, it's not possible that new blocks
             # get created at this point and we can safely delete the inode
+            self.db.execute('UPDATE names SET refcount = refcount - 1 WHERE '
+                            'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
+                            (id_new,))
+            self.db.execute('DELETE FROM names WHERE refcount=0')                 
             self.db.execute('DELETE FROM ext_attributes WHERE inode=?', (id_new,))
             self.db.execute('DELETE FROM symlink_targets WHERE inode=?', (id_new,))
             del self.inodes[id_new]
