@@ -12,7 +12,8 @@ from contextlib import contextmanager
 from s3ql.backends import local
 from s3ql.backends.common import BucketPool, AbstractBucket
 from s3ql.block_cache import BlockCache
-from s3ql.common import create_tables, init_tables
+from s3ql.mkfs import init_tables
+from s3ql.metadata import create_tables
 from s3ql.database import Connection
 import llfuse
 import os
@@ -356,7 +357,11 @@ class TestBucketPool(AbstractBucket):
         """
         return self.bucket.rename(src, dest)
 
-
+    def get_size(self, key):
+        '''Return size of object stored under *key*'''
+        
+        return self.bucket.get_size(key) 
+    
 def commit(cache, inode, block=None):
     """Upload data for `inode`
     
