@@ -14,9 +14,12 @@ from s3ql import fs
 from s3ql.backends import local
 from s3ql.backends.common import BucketPool
 from s3ql.block_cache import BlockCache
-from s3ql.common import ROOT_INODE, create_tables, init_tables
+from s3ql.common import ROOT_INODE
+from s3ql.mkfs import init_tables
+from s3ql.metadata import create_tables
 from s3ql.database import Connection
 from s3ql.fsck import Fsck
+from s3ql.inode_cache import InodeCache
 import errno
 import llfuse
 import os
@@ -64,7 +67,8 @@ class fs_api_tests(TestCase):
         
         self.block_cache = BlockCache(self.bucket_pool, self.db, self.cachedir + "/cache",
                                       self.blocksize * 5)
-        self.server = fs.Operations(self.block_cache, self.db, self.blocksize)
+        self.server = fs.Operations(self.block_cache, self.db, self.blocksize,
+                                    InodeCache(self.db, 0))
           
         self.server.init()
 
@@ -473,7 +477,28 @@ class fs_api_tests(TestCase):
         self.assertEqual(self.server.getxattr(inode.id, 'my-attr'), 'strabumm!')
 
         self.fsck()
+        
+    def test_names(self):
+        name1 = self.newname()
+        name2 = self.newname()
 
+        (fh, _) = self.server.create(ROOT_INODE, name1, self.file_mode(), Ctx())
+        self.server.release(fh)
+                
+        (fh, inode) = self.server.create(ROOT_INODE, name2, self.file_mode(), Ctx())
+        self.server.release(fh)
+
+        self.server.setxattr(inode.id, name1, 'strabumm!')     
+        self.fsck()
+        
+        self.server.removexattr(inode.id, name1)
+        self.fsck()
+        
+        self.server.setxattr(inode.id, name1, 'strabumm karacho!!')     
+        self.server.unlink(ROOT_INODE, name1)
+        self.fsck()
+                
+        
     def test_statfs(self):
         # Test with zero contents
         self.server.statfs()
@@ -613,7 +638,9 @@ class fs_api_tests(TestCase):
         self.fsck()
         
     def test_copy_tree(self):
-
+        ext_attr_name = 'system.foo.brazl'
+        ext_attr_val = 'schulla dku woumm bramp'
+        
         src_inode = self.server.mkdir(ROOT_INODE, 'source', self.dir_mode(), Ctx())
         dst_inode = self.server.mkdir(ROOT_INODE, 'dest', self.dir_mode(), Ctx())
 
@@ -622,6 +649,7 @@ class fs_api_tests(TestCase):
                                             self.file_mode(), Ctx())
         self.server.write(fh, 0, 'file1 contents')
         self.server.release(fh)
+        self.server.setxattr(f1_inode.id, ext_attr_name, ext_attr_val)
 
         # Create hardlink
         (fh, f2_inode) = self.server.create(src_inode.id, 'file2',
@@ -662,6 +690,8 @@ class fs_api_tests(TestCase):
         self.assertEqual(self.server.read(fh, 0, 42), 'file1 contents')
         self.server.release(fh)
         self.assertNotEqual(f1_inode.id, f1_inode_c.id)
+        self.assertEqual(self.server.getxattr(f1_inode_c.id, ext_attr_name), 
+                         ext_attr_val)
 
         # Check file2
         fh = self.server.open(f2_inode_c.id, os.O_RDWR)
@@ -678,6 +708,21 @@ class fs_api_tests(TestCase):
         
         self.fsck()
 
+    def test_copy_tree_2(self):   
+        src_inode = self.server.mkdir(ROOT_INODE, 'source', self.dir_mode(), Ctx())
+        dst_inode = self.server.mkdir(ROOT_INODE, 'dest', self.dir_mode(), Ctx())
+
+        # Create file
+        (fh, _) = self.server.create(src_inode.id, 'file1',
+                                     self.file_mode(), Ctx())
+        self.server.write(fh, 0, 'block 1 contents')
+        self.server.write(fh, self.blocksize, 'block 1 contents')
+        self.server.release(fh)
+
+        self.server.copy_tree(src_inode.id, dst_inode.id)
+        
+        self.fsck()
+        
     def test_lock_tree(self):
 
         inode1 = self.server.mkdir(ROOT_INODE, 'source', self.dir_mode(), Ctx())
